import java.io.*;
import java.util.*;

/** La classe terminal permet de raliser ses premiers programmes Java en permettant d'afficher dans la console d'excution des donnes de type diffrents, et en permettant de saisir au clavier des donnes de type diffrents.<BR>
    Elle permet aussi de lire et crire un fichier texte
    Cette classe contient que des mthodes statiques. */
public class Terminal{    

    // Le buffer standard  de lecture = le clavier
    private static BufferedReader in =
        new BufferedReader(new InputStreamReader(System.in));

    /** Cette mthode lit un fichier texte et retourne un le contenu du fichier sous la forme d'un StringBuffer.
        @param nomFichier le nom du fichier qui doit tre dans le rpertoire courant.
        @return StringBuffer le contenu du fichier. 
        @exception TerminalException (de type RuntimeException) si erreur d'criture<BR>
        Rappel : Une exception de type RuntimeException n'a pas l'obligation d'tre capture.
*/
    public static StringBuffer lireFichier(String nomFichier)
    {
        try{
            File fichier = new File(nomFichier);
            FileInputStream fis = new FileInputStream(new File(nomFichier));
            
            byte[] buffer = new byte[(int)fichier.length()];
            fis.read(buffer);
            fis.close();
            return(new StringBuffer(new String(buffer)));
        }
        catch(Exception ex)
            {
                exceptionHandler(ex);
            }
        return null;
    }

    /** Cette mthode permet de crer un fichier texte  partir du contenu d'un StringBuffer.
        @param nomFichier Le nom du fichier qui est cr dans le rpertoire courant
        @param strbuf Le StringBuffer contenant le texte  crire. 
        @exception TerminalException (de type RuntimeException) si erreur d'criture
    */
    public static void ecrireFichier(String nomFichier,
                                     StringBuffer strbuf)
    {
        try{
            File fichier = new File(nomFichier);
            FileOutputStream fos = new FileOutputStream(new File(nomFichier));
            
            byte[] buffer = strbuf.toString().getBytes();
            fos.write(buffer);
            fos.close();
        }
        catch(Exception ex)
            {
                exceptionHandler(ex);
            }
    }

    /** Cette mthode lit une chane de caractre
        @return String la chane saisie dans la console d'excution
       @exception TerminalException (de type RuntimeException) si erreur de lecture
    */
    public static String lireString() // Lire un String
    {
        String tmp="";
        char C='\0';
        try {
            tmp = in.readLine();
        }
        catch (IOException e)
            {
                exceptionHandler(e);
            }
        return tmp;
    }

    /** Cette mthode lit un entier
        @return int L'entier saisi dans la console d'excution
       @exception TerminalException (de type RuntimeException) si la saisie n'est pas un entier ou erreur de lecture
    */
    public static int lireInt()  // Lire un entier
    {
        int x=0;
        try {
            x=Integer.parseInt(lireString());
        }
        catch (NumberFormatException e) {
            exceptionHandler(e);
        }	
        return x ;
    }

    /** Cette mthode lit un boolean (false ou true)
        @return boolean Le boolean saisi dans la console d'excution
       @exception TerminalException (de type RuntimeException) si erreur de lecture. <BR>
       Tout autre valeur que TRUE, FALSE, true ou false, retourne la valeur false
    */
    public static boolean lireBoolean()  // Lire un entier
    {
        boolean b = true;
        try {
            b = Boolean.valueOf(lireString()).booleanValue();
        }
        catch (NumberFormatException e) {
            exceptionHandler(e);
        }	
        return b;
    }

    /** Cette mthode lit un double
        @return double Le double saisi dans la console d'excution
       @exception TerminalException (de type RuntimeException) si la valeur saisie n'est pas un double ou ereur de lecture.
    */
    public  static double lireDouble()  // Lire un double
    {
        double x=0.0;
        try {
            x=Double.valueOf(lireString()).doubleValue();
        }
        catch (NumberFormatException e) {
            exceptionHandler(e);
        }	
        return x ;
    }

    /** Cette mthode lit un caractre.
       @exception TerminalException (de type RuntimeException) si erreur de lecture.<BR>
       Si on saisit plus d'1 caractre alors le caractre retourn est le premier.
    */
    public  static char lireChar()  // Lire un caractere
    {
        String tmp=lireString();
        if (tmp.length()==0)
            return '\n';
        else 
            {
                return tmp.charAt(0);
            }
    }

    /** Cette mthode ecrit une chaine et ne revient pas  la ligne.
    */
    public static void ecrireString(String s){ // Afficher un String
        System.out.print(s);
    }

    /** Cette mthode crit une chaine et revient  la ligne.
    */
    public static void ecrireStringln(String s) // Afficher un String
    {
        ecrireString(s);
        sautDeLigne();
    }

    /** Cette mthode ecrit un entier et ne revient pas  la ligne.
    */
    public static void ecrireInt(int i)  // Afficher un entier
    {
        ecrireString(""+i);
    }

    /** Cette mthode crit un entier et revient  la ligne.
    */
    public static void ecrireIntln(int i)  // Afficher un entier
    {
        ecrireString(""+i);
        sautDeLigne();
    }

    /** Cette mthode crit un boolan et ne revient pas  la ligne.
    */
    public static void ecrireBoolean(boolean b){
        ecrireString(""+b);
    }

    /** Cette mthode crit un boolan et revient  la line.
    */
    public static void ecrireBooleanln(boolean b){
        ecrireString(""+b);
        sautDeLigne();
    }

    /** Cette mthode crit un double et ne revient pas  la ligne.
    */
    public  static void ecrireDouble(double d)  // Afficher un double
    {
        ecrireString(""+d);
    }

    /** Cette mthode crit un double et revient  la ligne.
    */
    public  static void ecrireDoubleln(double d)  // Afficher un double
    {
        ecrireDouble(d);
        sautDeLigne();
    }

    /** Cette mthode crit un caractre et ne revient pas  la ligne.
    */
    public  static void ecrireChar(char c)  // Afficher un caractere
    {
        ecrireString(""+c);
    }  

    /** Cette mthode crit un caractre et revient  la ligne.
    */
    public  static void ecrireCharln(char c)  // Afficher un caractere
    {
        ecrireChar(c);
        sautDeLigne();
    }

    /** Cette mthode revient  la ligne.
    */
    public static void sautDeLigne(){
        try{
            System.out.println();
        }catch(Exception ex){
            exceptionHandler(ex);
        }
    }

    /** Cette mthode retourne l'exception TerminalException
    */
    protected static void exceptionHandler(Exception ex){
        TerminalException err = new TerminalException(ex);
        throw err;
    }

    /** Cette mthode crit l'exception
    */
    public static void ecrireException(Throwable ex){
        ecrireString(ex.toString());
        ex.printStackTrace(System.err);
    }
}  

/** Classe de dfinition de l'exception TerminalException qui peut tre retourne dans l'usage des mthodes de la classe Terminal. */
class TerminalException extends RuntimeException{
    Exception ex;
    TerminalException(Exception e){
        ex = e;
    }
}


